@@ -9,7 +9,7 @@ from secrets import token_urlsafe
 from shlex import shlex
 from shutil import move
 from tempfile import mkstemp
-from typing import Iterator, Optional, Union
+from typing import overload, Iterator, Literal, Optional, Union
 
 from . import db, email, utils
 
@@ -137,13 +137,13 @@ class EnvVar:
         yield self.key
         yield self.value
 
-    def __len__(self) -> int:
+    def __len__(self) -> Literal[0, 1, 2]:
         """
         Return a number which represents the state of the ``EnvVar``.
 
         Returns
         -------
-        int
+        Literal[0, 1, 2]
             ``0`` if the ``EnvVar`` is ``None``, ``1``
             if the value is blank, or ``2`` otherwise.
         """
@@ -226,7 +226,7 @@ class Env(PathLike):
         EnvError
             If the environment variable is missing.
         """
-        value = self.vars.get(key)
+        value = self.ENV.get(key, self.vars.get(key))
         if value is None:
             raise EnvError(f"Missing environment variable: '{key}'")
         return value
@@ -343,7 +343,7 @@ class Env(PathLike):
     def vars(self) -> dict[str, str]:
         """`dict[str, str]` : Get the environment variables as a ``dict``."""
         def _sub_callback(match):  # type: ignore
-            return (self.ENV | result).get(match.group(1), '')  # type: ignore
+            return (self.ENV | result).get(match.group(1), '')
 
         with open(self.envfile, 'r') as f:
             envvars = list(filter(EnvVar.__len__, map(EnvVar, f.readlines())))
@@ -358,7 +358,13 @@ class Env(PathLike):
 
     def setenv(self) -> None:
         """Add the variables defined in the dotenv file to :os:`environ`."""
-        self.ENV |= self.vars  # type: ignore
+        self.ENV |= self.vars
+
+    @overload
+    def get(self, key: str, default: str) -> str: ...
+
+    @overload
+    def get(self, key: str, default: None = None) -> Optional[str]: ...
 
     def get(self, key: str, default: Optional[str] = None) -> Optional[str]:
         """
@@ -383,6 +389,12 @@ class Env(PathLike):
         """
         return self.ENV.get(key, self.vars.get(key) or default)
 
+    @overload
+    def bool(self, key: str, default: bool) -> bool: ...
+
+    @overload
+    def bool(self, key: str, default: None = None) -> Optional[bool]: ...
+
     def bool(self, key: str, default: Optional[bool] = None) -> Optional[bool]:
         """
         Return an environment variable as a ``bool``, or a default value.
@@ -418,6 +430,12 @@ class Env(PathLike):
             return False
         raise EnvError(f"Invalid boolean value: '{value}'")
 
+    @overload
+    def int(self, key: str, default: int) -> int: ...
+
+    @overload
+    def int(self, key: str, default: None = None) -> Optional[int]: ...
+
     def int(self, key: str, default: Optional[int] = None) -> Optional[int]:
         """
         Return an environment variable as an ``int``, or a default value.
@@ -452,6 +470,12 @@ class Env(PathLike):
         except ValueError:
             raise EnvError(f"Invalid integer value: '{value}'")
 
+    @overload
+    def float(self, key: str, default: float) -> float: ...
+
+    @overload
+    def float(self, key: str, default: None = None) -> Optional[float]: ...
+
     def float(self, key: str, default:
               Optional[float] = None) -> Optional[float]:
         """
@@ -487,6 +511,13 @@ class Env(PathLike):
         except ValueError:
             raise EnvError(f"Invalid numerical value: '{value}'")
 
+    @overload
+    def list(self, key: str, default: list, separator: str = ...) -> list: ...
+
+    @overload
+    def list(self, key: str, default: None = None,
+              separator: str = ...) -> Optional[list]: ...
+
     def list(self, key: str, default: Optional[list] = None,
              separator: str = ',') -> Optional[list]:
         """
@@ -516,6 +547,12 @@ class Env(PathLike):
             return default
         return value.split(separator)
 
+    @overload
+    def db(self, key: str, default: str) -> db.DBConfig: ...
+
+    @overload
+    def db(self, key: str, default: None = None) -> Optional[db.DBConfig]: ...
+
     def db(self, key: str, default:
            Optional[str] = None) -> Optional[db.DBConfig]:
         """
@@ -550,6 +587,13 @@ class Env(PathLike):
         except Exception as e:
             raise EnvError(f"Invalid database URL: '{value}'") from e
 
+    @overload
+    def email(self, key: str, default: str) -> email.EmailConfig: ...
+
+    @overload
+    def email(self, key: str, default: None = None
+              ) -> Optional[email.EmailConfig]: ...
+
     def email(self, key: str, default:
               Optional[str] = None) -> Optional[email.EmailConfig]:
         """
